HyperLogLogPlus

Struct HyperLogLogPlus 

pub struct HyperLogLogPlus<H, B>
where H: Hash + ?Sized, B: BuildHasher,
{ /* private fields */ }
Implements the HyperLogLog++ algorithm for cardinality estimation.

This implementation is based on the paper:

HyperLogLog in Practice: Algorithmic Engineering of a State of The Art Cardinality Estimation Algorithm.

  • Uses 6-bit registers, packed in a 32-bit unsigned integer. Thus, every five registers 2 bits are not used.
  • In small cardinalities, a sparse representation is used which allows for higher precision in estimations.
  • Performs bias correction using the empirical data provided by Google (can be found here).
  • Supports serialization/deserialization through serde.

Examples

use std::collections::hash_map::RandomState;
use hyperloglogplus::{HyperLogLog, HyperLogLogPlus};

let mut hllp: HyperLogLogPlus<u32, _> =
    HyperLogLogPlus::new(16, RandomState::new()).unwrap();

hllp.insert(&12345);
hllp.insert(&23456);

assert_eq!(hllp.count().trunc() as u32, 2);

References

Implementations

Source

impl<H, B> HyperLogLogPlus<H, B>
where H: Hash + ?Sized, B: BuildHasher,

pub fn new(precision: u8, builder: B) -> Result<Self, HyperLogLogError>

Creates a new HyperLogLogPlus instance.

pub fn merge<S, T>( &mut self, other: &HyperLogLogPlus<S, T>, ) -> Result<(), HyperLogLogError>
where S: Hash + ?Sized, T: BuildHasher,

Merges the other HyperLogLogPlus instance into self.

Both sketches must have the same precision. Merge can trigger the transition from sparse to normal representation.

pub fn insert_any<R>(&mut self, value: &R)
where R: Hash + ?Sized,

Inserts a new value, of any type, to the multiset.